<?php

/**
 * Walker class for nav menus. We extend Walker_Nav_Menu in case anything type
 * checks the walker class. We take the original walker as a constructor arg and
 * defer method calls to the original walker if the menu item is visible.
 *
 * This new approach should allow us to be compatible with code or themes that
 * implement their own walker classes.
 *
 * @author Brandon Wamboldt <brandon.wamboldt@gmail.com>
 */
class WpacSecureWalker extends Walker_Nav_Menu
{
  /**
   * @var boolean
   */
  protected $in_private = false;

  /**
   * @var integer
   */
  protected $private_depth = 0;

  /**
   * @var Walker_Nav_Menu
   */
  protected $original_walker;

  /**
   * Constructor.
   *
   * @param Walker_Nav_Menu $original_walker
   */
  public function __construct($original_walker = null)
  {
    if (empty($original_walker)) {
      $original_walker = new Walker_Nav_Menu;
    }

    $this->original_walker = $original_walker;
    $this->sync_vars_with_original_walker();
  }

  /**
   * Sync public variables to the original walker.
   */
  public function sync_vars_with_original_walker()
  {
    $this->original_walker->tree_type = $this->tree_type;
    $this->original_walker->db_fields = $this->db_fields;
    $this->original_walker->max_pages = $this->max_pages;
  }

  /**
   * {@inheritdoc}
   */
  public function display_element($element, &$children_elements, $max_depth, $depth, $args, &$output)
  {
    $this->sync_vars_with_original_walker();

    if (WordPressAccessControl::check_conditions($element->object_id) || get_option('wpac_show_in_menus', 'with_access') == 'always') {
      $this->original_walker->display_element($element, $children_elements, $max_depth, $depth, $args, $output);
    } else {
      $children_elements[$element->{$this->db_fields['id']}] = array();
    }
  }

  /**
   * Divert all other method calls directly to the original walker class.
   *
   * @param  string $method
   * @param  array  $arguments
   * @return mixed
   */
  public function __call($method, $arguments)
  {
    $this->sync_vars_with_original_walker();
    return call_user_func_array(array($this->original_walker, $method), $arguments);
  }

  /**
   * Divert all property access requests to the original walker class.
   *
   * @param  string $var
   * @return mixed
   */
  public function __get($var)
  {
    return $this->original_walker->{$var};
  }

  /**
   * Divert all property access requests to the original walker class.
   *
   * @param  string $var
   * @return mixed
   */
  public function __set($var, $value)
  {
    $this->original_walker->{$var} = $value;
  }

  /**
   * Divert all property access requests to the original walker class.
   *
   * @param  string $var
   * @return mixed
   */
  public function __isset($var)
  {
    return isset($this->original_walker->{$var});
  }
}
